library(stockassessment)
load("run/model.RData")

N.SIM <- 1001  # set the number of iterations the the stochastic forecasts   
# Note - the number of iterations 10001, up from 1001. All are stable if seed is set - standard "12345" used.
# set to 101 for quick checking
# set to 1001 for routine checking
# set to 10001 preference

###############################
## Reference points  
#= WKCELTIC 2020
F_MSY <- 0.353        # 0.312 		# Median point estimates of (F05) EqSim with combined SR
F_MSY_lower <- 0.221  # 0.206	    # Median lower point estimates of (F05) EqSim with combined SR
F_MSY_upper <- 0.521  # 0.456 	  	# Median upper point estimates of (F05) EqSim with combined SR
Fpa <- 0.708          # 0.92        # F05.   # was Fpa  1.01 in first 2020  Flim combined with the assessment error
Flim <- 1.40          # 1.28      	# F with 50% probability of SSB less than Blim
Blim <- 9227          # 9420      	# Lowest SBB with above-average recruitment
Bpa <- 12822          # 13089     	# Blim combined with the assessment error
MSY_Btrigger <- 12822 # 13089 		# Bpa

###############################
## TAC & advice for previous year

prev.advice <- 4644
#prev.TAC <- 11901 #not needed

## catch in last data year
cur.cat <- rev(catchtable(fit,obs.show=T))[1]


###############################
## Forecast Assumptions

first.yr <-1993 # min(fit$data$years)  # First year in the assessment (intermediate year) for help estimating intermediate year Recruitment 
final.yr <-max(fit$data$years)  # Final year in the assessment (intermediate year) for help estimating intermediate year Recruitment 
av.yrs <- final.yr+(-2:0)  # final.yr   # Use last three years for mean weights & L-D partition  ##  final.yr+(-2:0) # this is last three years, so five years would be: Ay <- lastyear +(-4:0)
#R.range.yrs <-seq(first.yr,final.yr,1)  # Resample R from previous years, start of the time series to end of time series (median is calculated)
## alternatively sample from the n lowest observed years
n <- 8 #in 2025 this is the 25th percentile (8/32)
n <- 3 # three lowest values (ordinal logistic regression shows 66% likelihood that next recruitment will be in the bottom 4 rank)
r <- rectable(fit)
R.range.yrs <- sort(as.numeric(rownames(r[order(r[,1])[1:n],])))


F_last <- tail(fbartable(fit),1)[,"Estimate"]  
## Fsq can be mean of last three years:
F_sq <- mean(tail(fbartable(fit),3)[,"Estimate"])
## or for scaled F (use selection pattern for last 3 years but rescale to the Fbar from last year_
#F_sq <- F_last

################################
## initial run to get SSB at the start of the advice year
## include the next line on pc to make the random number generator consistent with stockassessment.org
#RNGkind(sample.kind = "Rounding") 
set.seed(12345)
FC0 <- forecast(fit,fval=c(F_last,F_sq,F_MSY, F_MSY),ave.years=av.yrs, rec.years=R.range.yrs, splitLD=TRUE, nosim = N.SIM)
SSBNext <- attr(FC0,"shorttab")['ssb',3]

F_MSY_approach <- F_MSY*pmin(1,SSBNext/MSY_Btrigger)

###############################
## forecast names and structure part 1
## Forecasts

scen <- list(
  "MSY approach: FMSY" = list(fval = c(F_last, F_sq, F_MSY_approach, F_MSY)),
  "MSY: FMSY" = list(fval = c(F_last, F_sq, F_MSY, F_MSY)),
  "FMSY lower" = list(fval = c(F_last, F_sq, F_MSY_lower, F_MSY)),
  "FMSY upper" = list(fval = c(F_last, F_sq, F_MSY_upper, F_MSY)),
  "F = 0" = list(fval = c(F_last, F_sq, 0.000001 ,0.000001)),
  "Fpa" = list(fval = c(F_last,F_sq, Fpa, F_MSY)),
  "Flim" = list(fval = c(F_last, F_sq, Flim, F_MSY)),
  "Blim" = list(fval = c(F_last, F_sq, NA,F_MSY), nextssb = c(NA, NA, Blim, NA)),
  "MSY Btrigger" = list(fval = c(F_last, F_sq, NA, F_MSY), nextssb = c(NA, NA, MSY_Btrigger, NA)),
  "hit Bpa" = list(fval = c(F_last, F_sq, NA, F_MSY), nextssb = c(NA, NA, Bpa, NA)) ,
  "Fsq"  = list(fval = c(F_last, F_sq, F_sq, F_MSY)),
  "SSB Y+2 = SSB Y+1"  = list(fval = c(F_last, F_sq, NA, F_MSY), nextssb = c(NA, NA, SSBNext, NA))
)

FC <- ""
FC <- vector("list", length(scen))
names(FC) <- names(scen)


for(i in seq(scen)){
  set.seed(12345)
  ARGS <- scen[[i]]
  ARGS <- c(ARGS,
            list(fit = fit, ave.years=av.yrs, rec.years=R.range.yrs, label=names(scen)[i], splitLD=TRUE, nosim = N.SIM,savesim=T))  
  FC[[i]] <- try(do.call(forecast, ARGS))
  if(class(FC[[i]])=='try-error') FC[[i]] <- FC[['F = 0']]
  
  print(paste0("forecast : ", "'", names(scen)[i], "'", " is complete"))
}


save(FC, file="run/forecast.RData")


###########
### some extra tables to get the data formatted for advice sheet

### interrim year assumptions
iy <- data.frame(Fy=F_sq
                 ,SSBy1=SSBNext
                 ,Ry=attr(FC0,"shorttab")['rec',2]
                 ,Ry1=attr(FC0,"shorttab")['rec',3]
                 ,Cy=attr(FC0,"shorttab")['catch',2]
                 ,Ly=attr(FC0,"shorttab")['Land',2]
                 ,Dy=attr(FC0,"shorttab")['Discard',2])
names(iy) <- c(paste0('F',final.yr+1),
  paste0('SSB',final.yr+2),
  paste0('Rec',final.yr+1),
  paste0('Rec',final.yr+2),
  paste0('Cat',final.yr+1),
  paste0('Lan',final.yr+1),
  paste0('Dis',final.yr+1))

### risk to blim

cs <- data.frame(catchTACyr = NA,
                 Landings = NA,
                 Discards = NA,
                 Ftotal = NA,
                 Fland = NA,
                 Fdiscard = NA,
                 ssbTACyrplus1 = NA,
                 ssbChange = NA,
                 adviceChange = NA,
                 Risk = NA)

# the 3rd year in the forecast is the advice year because in this stock the last year with data is included in the forecast
for (i in c(1:length(FC))){
  cs[i,1] <- attr(FC[[i]],"shorttab")[4,3]
  cs[i,2] <- attr(FC[[i]],"shorttab")[7,3]
  cs[i,3] <- attr(FC[[i]],"shorttab")[8,3]
  cs[i,4] <- attr(FC[[i]],"shorttab")[1,3]
  cs[i,5] <- attr(FC[[i]],"shorttab")[5,3]
  cs[i,6] <- attr(FC[[i]],"shorttab")[6,3]
  cs[i,7] <- attr(FC[[i]],"shorttab")[3,4]
  cs[i,8] <- 100*attr(FC[[i]],"shorttab")[3,4]/SSBNext-100
  cs[i,9] <- 100*attr(FC[[i]],"shorttab")[4,3]/prev.advice-100
}
cs[,10] <- 100*unlist(lapply(FC, function(x) sum(x[[4]]$ssb < Blim) / length(x[[4]]$ssb)))
rownames(cs) <- names(scen)
save(FC,iy,cs, file="run/forecast.RData")

# reference points (to make sure you are using the correct ones)
rp <- data.frame(Btrigger=MSY_Btrigger,Fmsy=F_MSY,Blim,Bpa=MSY_Btrigger,Flim,Fpa,Btrigger=MSY_Btrigger,Bpa,Fmsy=F_MSY,Flower=F_MSY_lower,Fupper=F_MSY_upper)
save(FC,iy,cs,rp, file="run/forecast.RData")
